### Persefone - a socio-economic-ecological model of European agricultural landscapes.
###
### This file is responsible for managing the crop growth modules.
###

@enum GrowthPhase janfirst sow marchfirst harvest1 harvest2 vegphase

"""
    CropType

The type struct for all crops. Currently follows the crop
growth model as implemented in ALMaSS.
"""
@kwdef struct CropType
    name::String
    sowdate::Date
    harvestdate::Date
    mingrowthtemp::Float64
    inflectionpoints::Dict{GrowthPhase,Vector{Float64}}
    slopes::Dict{GrowthPhase,Vector{Float64}}
    #issowable::Union{Function,Bool}=true
end

let croptypes = Dict{String,CropType}()
    global function registercrop(ct::CropType)
        croptypes[ct.name] = ct
    end

    global function getcrop(name::String)
        croptypes[name]
    end
end

"""
    @dday(datestring)

A utility macro to make it quicker to create dates in the
format "21 August".
"""
macro dday(date::String)
    :(Date($(date), dateformat"d U"))
end

let croplist = Dict{String,CropType}()
    
    global function newcrop(;kwargs...)
        crop = CropType(;kwargs...)
        croplist[crop.name] = crop
    end

    global function cropdefinition(name)
        croplist[name]
    end
end

#
# XXX It would be nicer to be able to define crops with a macro
# (like @species), but since macros are so tricky to get right,
# I'll leave that for later.
#
# """
#     @crop
#
# A macro to create and register a new crop type. Pass a list
# of keyword arguments to instantiate each field of the
# [`CropType`](@ref) struct.
# """
# macro crop(;kwargs...) # macros don't take kwargs
#     quote
#         registercrop(CropType($(esc(kwargs))))
#     end
# end
#
# @crop "winter wheat" begin
#     minsowdate = @dday("1 November")
#     maxsowdate = @dday("31 November")
#     growthparameter = 1.0
#     issowable = true
# end

newcrop(name="winter wheat",
        sowdate = @dday("1 November"),
        harvestdate = @dday("1 July"),
        mingrowthtemp = 5.0,
        growthparameter = 1.0,
        issowable = true)

